<?php

/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.3                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2005 m3 Media Services Limited                         |
| For contact details, see: http://www.m3.net/                              |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id: PriorityAdServerGetRequiredAdImpressionsType1.mtp.test.php 3416 2005-06-30 14:40:49Z andrew $
*/

require_once MAX_PATH . '/lib/max/Maintenance/Priority/AdServer/tasks/GetRequiredAdImpressionsType1.php';

/**
 * A class for testing the Maintenance_Priority_AdServer_RequiredAdvertisementImpression_Normal class.
 *
 * @package    MaxMaintenance
 * @subpackage TestSuite
 * @author     James Floyd <james@m3.net>
 */
class TestOfPriorityAdserverGetRequiredAdImpressionsType1 extends UnitTestCase
{
    
    /**
     * The constructor method.
     */
    function TestOfPriorityAdserverGetRequiredAdImpressionsType1()
    {        
        $this->UnitTestCase();
        Mock::generate('MAX_Dal_Maintenance');
        Mock::generate('MAX_Table_Priority');
    }
    
    function test_getMaxMaintenancePriorityCriteria()
    {
        $obj = & $this->getCurrentTask();
        $oReturn = $obj->_getMaxMaintenancePriorityCriteria(array());
        $this->assertTrue(is_a($oReturn, 'MAX_Maintenance_Priority_Criteria'));
    }
    
    function test_getMaxTablePriorityObj()
    {
        // Get instance of class
        $obj = & $this->getCurrentTask();
        // Test object of type MAX_Table_Priority is returned
        $this->assertTrue(is_a($obj->_getMaxTablePriorityObj(), 'MockMAX_Table_Priority'));
    }
    
    function test_getDate()
    {
        // Get instance of class
        $obj = & $this->getCurrentTask();
        
        // Test 1 - test return object type
        $oDate = $obj->_getDate();
        $oDateNow = new Date();
        $this->assertTrue(is_a($oDate, 'Date'));
        
        // Test 2 - test values set to date time now
        $this->assertTrue($oDate->getYear() == $oDateNow->getYear());
        $this->assertTrue($oDate->getMonth() == $oDateNow->getMonth());
        $this->assertTrue($oDate->getDay() == $oDateNow->getDay());
        $this->assertTrue($oDate->getHour() == $oDateNow->getHour());
        $this->assertTrue($oDate->getMinute() == $oDateNow->getMinute());
        $this->assertTrue($oDate->getSecond() == $oDateNow->getSecond());
        
        // Test 3 - test sspecific date time return
        $oDate = $obj->_getDate('2005-05-05 10:59:59');
        $this->assertTrue($oDate->getYear() == 2005);
        $this->assertTrue($oDate->getMonth() == 05);
        $this->assertTrue($oDate->getDay() == 05);
        $this->assertTrue($oDate->getHour() == 10);
        $this->assertTrue($oDate->getMinute() == 59);
        $this->assertTrue($oDate->getSecond() == 59);
    }
    
    
    //DT: i've disabled this method because it's more of a DAL test
    //... and i can't get it working with the service locator
    function xtestGetAllCampaigns()
    {
        Mock::generatePartial(
            'GetRequiredAdImpressionsType1', 
            $class = 'GetRequiredAdImpressionsType1' . rand(),
            array('_getDal')
        );
        
        $aCampaignFromDb = array(
            1 => array(
                'campaignid' => 1,
                'expire' => '2006-10-10 10:59:59',
                'clicks' => 1000,
                'conversions' => 10,
                'views' => 10000,
                'target' => 0,
                'priority' => 1,
            ),
            2 => array(
                'campaignid' => 2,
                'expire' => '2007-10-10 10:59:59',
                'clicks' => 222,
                'conversions' => 20,
                'views' => 20000,
                'target' => 10,
                'priority' => 2,
            )
        );
        // Create mock DAL
        $this->oDal->setReturnValue('getCampaigns', $aCampaignFromDb);
        // Create mock test class
        $obj = & new $class($this);
        // Register mock dal as return val
        $obj->setReturnValue('_getDal', $this->oDal);
        // Call constructor to populate DAL property with mock
        $obj->getRequiredAdImpressionsType1();
        // Call test method
        $aCampaigns = $obj->getAllCampaigns($aCampaignFromDb);
        // Check return - type of objects
        foreach ($aCampaigns as $oCampaign) {
            $this->assertTrue(is_a($oCampaign, 'Campaign'));
        }
        // Test first Campign object values
        $this->assertTrue($aCampaigns[0]->id == 1);
        $this->assertTrue($aCampaigns[0]->expire == '2006-10-10 10:59:59');
        $this->assertTrue($aCampaigns[0]->clicks == 1000);
        $this->assertTrue($aCampaigns[0]->conversions == 10);
        $this->assertTrue($aCampaigns[0]->views == 10000);
        $this->assertTrue($aCampaigns[0]->target == 0);
        $this->assertTrue($aCampaigns[0]->priority == 1);
        // Test second Campaign object values
        $this->assertTrue($aCampaigns[1]->id == 2);
        $this->assertTrue($aCampaigns[1]->expire == '2007-10-10 10:59:59');
        $this->assertTrue($aCampaigns[1]->clicks == 222);
        $this->assertTrue($aCampaigns[1]->conversions == 20);
        $this->assertTrue($aCampaigns[1]->views == 20000);
        $this->assertTrue($aCampaigns[1]->target == 10);
        $this->assertTrue($aCampaigns[1]->priority == 2);
    }
    
   /**
    * Test that the default impression to click ratio
    * and impression to conversion ratio settings
    *
    * Test 1 - make sure values exist
    * Test 2 - make sure values are > 0 and < 1
    */
    function testDefaultRatioDefined()
    {
        $conf = $GLOBALS['_MAX']['CONF'];
        #print '<pre>'; print_r($GLOBALS['_MAX']['CONF']); print '</pre>';
        // Test 1
        $this->assertNotEqual($conf['priority']['defaultClickRatio'], 0, 
            "Conf setting ['priority']['defaultClickRatio'] must have a value");
        $this->assertNotEqual($conf['priority']['defaultConversionRatio'], 0, 
            "Conf setting ['priority']['defaultConversionRatio'] must have a value");
        
        // Test 2
        $this->assertTrue(($conf['priority']['defaultClickRatio'] > 0) 
            && ($conf['priority']['defaultClickRatio'] < 1),
            "Conf setting ['priority']['defaultClickRatio'] must be between 0 and 1");
        
        $this->assertTrue(($conf['priority']['defaultConversionRatio'] > 0) 
            && ($conf['priority']['defaultConversionRatio'] < 1),
            "Conf setting ['priority']['defaultConversionRatio'] must be between 0 and 1");
    }
    
   /**
    * Test method that corretctly calculates
    * Test 1 - perform simple calculation test
    * Test 2 - test that impressions are being rounded up
    * Test 3 - test calculation when no historical data
    * Test 4 - test no inventory requirment
    */
    function testGetInventoryImpressionsRequired()
    {
        $conf = $GLOBALS['_MAX']['CONF'];
        $obj = & $this->getCurrentTask();
        
        // Test 1
        $inventory = 10;
        $inventoryToDate = 100;
        $impressionsToDate = 100000;
        
        $result = $obj->getInventoryImpressionsRequired(
                $inventory, $conf['priority']['defaultConversionRatio'],
                $inventoryToDate, $impressionsToDate);
        $this->assertEqual($result, 10000);
        
        // Test 2
        $inventory = 546245;
        $inventoryToDate = 65;
        $impressionsToDate = 45684;
        
        $result = $obj->getInventoryImpressionsRequired(
                $inventory, $conf['priority']['defaultConversionRatio'],
                $inventoryToDate, $impressionsToDate);
        $this->assertEqual($result, 383917794);
        
        // Test 3
        $inventory = 1000;
        $inventoryToDate = 0;
        $impressionsToDate = 0;
        
        $result = $obj->getInventoryImpressionsRequired(
                $inventory, $conf['priority']['defaultConversionRatio'],
                $inventoryToDate, $impressionsToDate);
        $this->assertEqual($result, (ceil((int)$inventory / $conf['priority']['defaultConversionRatio'])));
        
        // Test 4
        $inventory = 0;
        $inventoryToDate = 0;
        $impressionsToDate = 0;
        
        $result = $obj->getInventoryImpressionsRequired(
                $inventory, $conf['priority']['defaultConversionRatio'],
                $inventoryToDate, $impressionsToDate);
        $this->assertEqual($result, 0);
    }
       
   /**
    * Test that method sets property of mock Placement object 
    * $oPlacement->requiredImpressions to be calculated value
    * of required impressions
    */
    function testGetPlacementImpressionInventoryRequirement()
    {
        // Generate a partial mock of the Placement class
        Mock::generatePartial('Placement',
                              'PlacementInventoryRequirement',
                              array('setSummaryStatisticsToDate'));
        $oPlacement = &new PlacementInventoryRequirement($this);
        $oPlacement->setReturnValue('setSummaryStatisticsToDate', true);
        // Set remaining inventory
        $oPlacement->clicks      = 10;
        $oPlacement->conversions = 0;
        // Set historical data for campaign
        $oPlacement->sumConversions = 10;
        $oPlacement->sumClicks      = 100;
        $oPlacement->sumViews       = 10000;
        // Get getCurrentTask
        $obj = $this->getCurrentTask();
        $obj->getPlacementImpressionInventoryRequirement($oPlacement);
        $this->assertEqual(1000, $oPlacement->requiredImpressions);
    }
    
    /**
    * Test to see if function returns smallest number in array
    */
    function testGetSmallestNonZeroConstraint()
    {
        $obj = & $this->getCurrentTask();
        
        // Test selectSmallestNonZero() method return value
        $this->assertEqual(0, $obj->getSmallestNonZeroConstraint(array(0,0,0)), '');
        $this->assertEqual(1, $obj->getSmallestNonZeroConstraint(array(1,0,0)), '');
        $this->assertEqual(1, $obj->getSmallestNonZeroConstraint(array(-1,1,1)), '');
        $this->assertEqual(1, $obj->getSmallestNonZeroConstraint(array(1,1,1)), '');
        $this->assertEqual(0, $obj->getSmallestNonZeroConstraint(array(-1,-1,-1)), '');
        $this->assertEqual(4, $obj->getSmallestNonZeroConstraint(array(32,18,4)), '');
        $this->assertEqual(748, $obj->getSmallestNonZeroConstraint(array(748,849,35625)), '');
        $this->assertEqual(0, $obj->getSmallestNonZeroConstraint(array(-1,'what','string')), '');
        $this->assertEqual(0, $obj->getSmallestNonZeroConstraint('foo'), '');
        $this->assertEqual(0, $obj->getSmallestNonZeroConstraint(5000), '');
        $this->assertEqual(0, $obj->getSmallestNonZeroConstraint(), '');
    }
    
    /* Test using integration tests */
    function testDistributeCampaignsImpressions()
    {

    }
    
//    function getTmpAdRequiredImpression() {
//        $aColumns = array('ad_id' => 'ad_id', 'required_impressions' => 'required_impressions');
//        $aTable = array('tmp_ad_required_impression' => 'tmp_ad_required_impression');
//        $aLimitations = 'ad_id is not null';
//        $aGroupColumns = array();
//        $primaryKey = array();
//        return SqlBuilder::_select($aColumns, $aTable, $aLimitations, $aGroupColumns, 'ad_id');
//    }
    
    /**
    * Test to see if method is summing ad weights correctly
    */
    function testGetCampaignAdWeightTotal_sum()
    {   
        // Create some test advert
        $oAdvert1 = new Advert(array('ad_id' => 1, 'weight' => 2));
        $oAdvert2 = new Advert(array('ad_id' => 2, 'weight' => 1));
        $oAdvert3 = new Advert(array('ad_id' => 3, 'weight' => 0));
        $oAdvert4 = new Advert(array('ad_id' => 4, 'weight' => 3));
        $oAdvert5 = new Advert(array('ad_id' => 5, 'weight' => -10));
        $oPlacement = &new Placement();
        $oPlacement->Placement();
        // Add the Adverts to the Placement
        $oPlacement->adverts[] = $oAdvert1;
        $oPlacement->adverts[] = $oAdvert2;
        $oPlacement->adverts[] = $oAdvert3;
        $oPlacement->adverts[] = $oAdvert4;
        $oPlacement->adverts[] = $oAdvert5;
        // Test
        $normal = &$this->getCurrentTask();
        $this->assertEqual(6, $normal->getPlacementAdWeightTotal($oPlacement));
    }
    
   /**
    * Test to see if method returns zero if no adverts
    * returner as resolut of $oCampaign->getAdverts()
    */
    function testGetCampaignAdWeightTotal_no_adverts()
    {           
        Mock::generatePartial('Placement', 'CampaignTestObj2', array('getAdverts'));
        
        $oPlacement = &new CampaignTestObj2($this);
        $oPlacement->setReturnValue('getAdverts', array());
        
        $normal = &$this->getCurrentTask();
        $this->assertEqual($normal->getPlacementAdWeightTotal($oPlacement), 1);
    }

    function &getCurrentTask()
    {
        $oDal = &new MockMAX_Dal_Maintenance($this);
        $oTable = &new MockMAX_Table_Priority($this);
        $oServiceLocator = &ServiceLocator::instance();
        $oServiceLocator->register('MAX_Dal_Maintenance', $oDal);
        $oServiceLocator->register('MAX_Table_Priority', $oTable);
        
        return new GetRequiredAdImpressionsType1();
    }
}
?>
